﻿using System;
using System.Runtime.InteropServices;

namespace KbdLEDs
{
    ////////////////////////////////////////////////////////////////////////////////
    // Switching Keyboard LEDs by toggling NumLock, CapsLock and ScrollLock
    // No Forms or WPF References used
    // © 2014 JoAchim Berg, Hamburg, Germany, achim_berg_de@yahoo.com
    // obs: CapsLock reset does not work if CapsLock On/Off uses different keys (driver-specific)
    ////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Keyboard LEDs enum
    /// </summary>
    public enum LED
    {
        ScrollLock  = 0x91,
        CapsLock    = 0x14,
        NumLock     = 0x90
    }

    /// <summary>
    /// Toggle, Set and Reset Keyboard Toggles
    /// </summary>
    public static class LEDs
    {
        private static System.Timers.Timer TScroll, TCaps, TNum;

        #region Basics

        /// <summary>
        /// Toggle one Led
        /// </summary>
        public static void Toggle(LED led)
        {
            keybd_event((byte)led, 0x45, 1, UIntPtr.Zero);
            keybd_event((byte)led, 0x45, 2, UIntPtr.Zero);
        }

        /// <summary>
        /// Set or reset "toggled" state of LED
        /// </summary>
        public static void SetToggleState(LED led, bool toggled = false)
        {
            if (GetToggleState(led) != toggled) Toggle(led);
        }

        /// <summary>
        /// Get current "toggled" state of Led 
        /// Forms Version
        /// </summary>
        public static bool GetToggleState(LED led)
        {
            switch (led)
            {
                case LED.ScrollLock:
                    return (((ushort)GetKeyState(0x91)) & 0x0001) != 0;

                case LED.NumLock:
                    return Console.NumberLock;

                case LED.CapsLock:
                    return Console.CapsLock;
            }
            return false;
        }

        /// <summary>
        /// Switches Led on for "msDuration" milliseconds
        /// </summary>
        public static void Impulse(LED led, int msDuration = 200)
        {
            if (msDuration <= 0) return;
            switch (led)
            {
                case LED.ScrollLock:
                    startScrollImpulse(msDuration);
                    break;

                case LED.CapsLock:
                    startCapsImpulse(msDuration);
                    break;

                case LED.NumLock:
                    startNumImpulse(msDuration);
                    break;
            }
        }

        #endregion

        #region Helpers

        private static void startScrollImpulse(int msDuration)
        {
            if (TScroll == null)
            {
                TScroll = new System.Timers.Timer();
                TScroll.Elapsed += ScrollTick;
            }
            else if (TScroll.Enabled)
            {
                TScroll.Enabled = false;
            }
            TScroll.Interval = msDuration;
            TScroll.Enabled = true;
            SetToggleState(LED.ScrollLock, true);
        }

        private static void ScrollTick(object sender, System.Timers.ElapsedEventArgs e)
        {
            TScroll.Enabled = false;
            SetToggleState(LED.ScrollLock, false);
        }

        private static void startCapsImpulse(int msDuration)
        {
            if (TCaps == null)
            {
                TCaps = new System.Timers.Timer();
                TCaps.Elapsed += CapsTick;
            }
            else if (TCaps.Enabled)
            {
                TCaps.Enabled = false;
            }
            TCaps.Interval = msDuration;
            TCaps.Enabled = true;
            SetToggleState(LED.CapsLock, true);
        }

        private static void CapsTick(object sender, System.Timers.ElapsedEventArgs e)
        {
            TCaps.Enabled = false;
            SetToggleState(LED.CapsLock, false);
        }

        private static void startNumImpulse(int msDuration)
        {
            if (TNum == null)
            {
                TNum = new System.Timers.Timer();
                TNum.Elapsed += NumTick;
            }
            else if (TNum.Enabled)
            {
                TNum.Enabled = false; 
            }
            TNum.Interval = msDuration;
            TNum.Enabled = true;
            SetToggleState(LED.NumLock, true);
        }

        private static void NumTick(object sender, System.Timers.ElapsedEventArgs e)
        {
            TNum.Enabled = false;
            SetToggleState(LED.NumLock, false);
        }

        #endregion

        [DllImport("user32.dll", EntryPoint = "keybd_event")]
        static extern void keybd_event(byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo);

        [DllImport("user32.dll", EntryPoint = "GetKeyState")]
        public static extern short GetKeyState(int keyCode);
    }
}
